// ConvertCP.cpp : Implementation of CConvertCP
#include "stdafx.h"
#include "CPConvert.h"
#include "ConvertCP.h"

/////////////////////////////////////////////////////////////////////////////
// CConvertCP

STDMETHODIMP CConvertCP::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IConvertCP
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CConvertCP::Convert(int nCPFrom, int nCPTo, BSTR strFrom, LPBSTR strOut)
{
	int nLenDest = 0;
	int nLen = 0;
	_TCHAR szMsg[_MAX_PATH*2];
	LPWSTR lpszTxtUni = NULL;
	LPSTR lpStrOut = NULL;
	LPSTR lpzsFrom = NULL;
	HRESULT hr = E_FAIL;
	DWORD dwCodeErr = 0;
	DWORD dwCode = 0;
	DWORD dwCurSize = 0;
	BOOL bEmpty = TRUE;
	szMsg[0] = (_TCHAR)'\0';

	if (!IsValidCodePage((UINT)nCPFrom))
	{
		dwCodeErr = ::GetLastError();
		_stprintf(szMsg, _T("Not valid CodePage From = %d"), nCPFrom);
		return Error(szMsg, IID_IConvertCP, HRESULT_FROM_WIN32(dwCodeErr));
	}
	if (!IsValidCodePage((UINT)nCPTo))
	{
		dwCodeErr = ::GetLastError();
		_stprintf(szMsg, _T("Not valid CodePage To = %d"), nCPTo);
		return Error(szMsg, IID_IConvertCP, HRESULT_FROM_WIN32(dwCodeErr));
	}
	if (strOut == NULL)
		return Error(L"Not valid arg 'strOut'", IID_IConvertCP, E_INVALIDARG);

	try
	{
		///////////////////////////////////////////////////////////////
		// Convert to Unicode (from code page nCPFrom)
		//
		size_t nSizeInp = SysStringLen(strFrom)*sizeof(WCHAR);
		if (nSizeInp)
		{
			bEmpty = FALSE;
			lpzsFrom = _com_util::ConvertBSTRToString(strFrom);
			if (lpzsFrom == NULL)
			{
				dwCodeErr = ::GetLastError();
				throw _T("Error out of memory for unicode from [ConvertBSTRToString()]");
			}
			//
			// Get WideChar buffer size for input string
			int nLenInp = 0;
			if ((nLenInp = MultiByteToWideChar(
					(UINT)nCPFrom,				// code page (from)
					0,							// character-type options 
					(LPCSTR)lpzsFrom,			// address of string to map
					nSizeInp,					// number of bytes in string
					NULL,						// address of wide-character buffer 
					0							// size of buffer 
				)) == 0) 
			{
				dwCodeErr = ::GetLastError();
				throw _T("Error MultiByteToWideChar() [get buffer size]");
			}

			// Check WideChar buffer for input string
			if ((dwCurSize = (DWORD)(++nLenInp)) > m_dwWideCharsBuff)
			{
				if ((dwCode = ReAllocBuffer(dwCurSize, WCBtype)) != NO_ERROR)
				{
					dwCodeErr = dwCode;
					throw _T("Error ReAllocate WCB buffer");
				}
			}
			lpszTxtUni = m_lpszWideCharsBuff;
			_ASSERTE(lpszTxtUni != NULL);

			//
			// Convert input string to WideChar buffer
			if ((nLen = MultiByteToWideChar(
					(UINT)nCPFrom,				// code page (from)
					0,							// character-type options 
					(LPCSTR)lpzsFrom,			// address of string to map (const char*)bstrFrom
					nSizeInp,					// number of bytes in string
					lpszTxtUni,					// address of wide-character buffer 
					nLenInp						// size of buffer 
				)) == 0) 
			{
				dwCodeErr = ::GetLastError();
				throw _T("Error MultiByteToWideChar() [convert]");
			}
			*(lpszTxtUni+nLen) = (WCHAR)'\0';
			// Ok!

			/////////////////////////////////////////////////////////
			// Convert from Unicode (to code page nCPTo)
			//
			LPCSTR lpzsDefaultChar = "?";
			BOOL bUsedDefaultChar = FALSE;
			//
			// Get MultiByte buffer size for output string
			int nSizeOut = 0;
			if ((nSizeOut = WideCharToMultiByte(
					(UINT)nCPTo,			// code page (to)
					0,						// performance and mapping flags 
					lpszTxtUni,				// address of wide-character string 
					nLen,					// number of characters in string 
					NULL,					// address of buffer for new string 
					0,						// size of buffer 
					lpzsDefaultChar,		// address of default for unmappable characters 
					&bUsedDefaultChar		// address of flag set when default char. used 
 				)) == 0) 
			{
				dwCodeErr = ::GetLastError();
				throw _T("Error WideCharToMultiByte() [get buffer size]");
			}
			
			// Check MultiByte buffer for output string
			if ((dwCurSize = (DWORD)(++nSizeOut)) > m_dwMultiBytesBuff)
			{
				if ((dwCode = ReAllocBuffer(dwCurSize, MBBtype)) != NO_ERROR)
				{
					dwCodeErr = dwCode;
					throw _T("Error ReAllocate MBB buffer");
				}
			}
			lpStrOut = m_lpszMultiBytesBuff;
			_ASSERTE(lpStrOut != NULL);

			//
			// Convert WideChar -> MultiByte buffer for output string
			if ((nLenDest = WideCharToMultiByte(
					(UINT)nCPTo,			// code page (to)
					0,						// performance and mapping flags 
					lpszTxtUni,				// address of wide-character string 
					nLen,					// number of characters in string 
					lpStrOut,				// address of buffer for new string 
					nSizeOut,				// size of buffer 
					lpzsDefaultChar,		// address of default for unmappable characters 
					&bUsedDefaultChar		// address of flag set when default char. used 
 				)) == 0) 
			{
				dwCodeErr = ::GetLastError();
				throw _T("Error WideCharToMultiByte() [convert]");
			}
			*(lpStrOut+nLenDest) = (CHAR)'\0';
			// Ok!
		}
	}
	catch (LPCTSTR lpszMsg)
	{
		if (dwCodeErr != 0)
		{
			LPVOID lpMsgBuf;
			::FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				dwCodeErr,
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR)&lpMsgBuf,
				0,
				NULL 
			);
			_stprintf(szMsg, _T("(%ld) %s : %s"), dwCodeErr, lpszMsg, lpMsgBuf);
			::LocalFree(lpMsgBuf);
		}
		else
			_stprintf(szMsg, _T("%s"), lpszMsg);
	}
	catch (...)
	{
		if (dwCodeErr == NO_ERROR)
			dwCodeErr = ::GetLastError();
		_tcscpy(szMsg, _T("Unexpected Error"));
	}

	// Clear memory
	if (lpzsFrom != NULL)
	{
		delete [] lpzsFrom;
		lpzsFrom = NULL;
	}
	lpszTxtUni = NULL;
	
	size_t nLenMsg = _tcslen(szMsg);
	BOOL bIsResult = FALSE;
	if ((lpStrOut != NULL || bEmpty) && nLenMsg == 0)
	{
		// Set result if no any errors
		if (bEmpty)
			hr = ::SysReAllocString(strOut, L"");
		else
		{
			USES_CONVERSION;
			hr = ::SysReAllocString(strOut, A2COLE(lpStrOut));
		}
		bIsResult = TRUE;
	}
	lpStrOut = NULL;

	// Check error on set result
	if (bIsResult && !SUCCEEDED(hr))
		return Error(L"Error on SysReAllocString() for result", IID_IConvertCP, hr);
	
	if (nLenMsg)
	{
		return Error(szMsg, IID_IConvertCP, HRESULT_FROM_WIN32(dwCodeErr));
	}
	
	if (bIsResult)
		return hr;
	else
		return E_FAIL;
}

HRESULT CConvertCP::FinalConstruct()
{
	DWORD dwCodeErr = CreateBuffers();
	if (dwCodeErr == NO_ERROR)
		return S_OK;
	else 
		return Error(L"Can not create buffers", IID_IConvertCP, HRESULT_FROM_WIN32(dwCodeErr));
}

void CConvertCP::FinalRelease()
{
	DWORD dwCodeErr = ReleaseBuffers();
	_ASSERTE(dwCodeErr == 0);
}

DWORD CConvertCP::CreateBuffers(DWORD dwWCBsize, DWORD dwMBBsize)
{
	DWORD dwRetCode = 0;
	if (m_hWideCharsBuff == NULL)
	{
		if ((m_hWideCharsBuff = ::GlobalAlloc(MEMTYPE, dwWCBsize)) == NULL)
		{
			dwRetCode = ::GetLastError();
		}
		else
		{
			_ASSERTE(m_lpszWideCharsBuff == NULL);
			if ((m_lpszWideCharsBuff = (LPWSTR)::GlobalLock(m_hWideCharsBuff)) == NULL)
			{
				dwRetCode = ::GetLastError();
			}
			else
			{
				m_dwWideCharsBuff = ::GlobalSize(m_hWideCharsBuff);
				_ASSERTE(m_dwWideCharsBuff != 0);
			}
			_ASSERTE(m_lpszWideCharsBuff != NULL);
		}

		_ASSERTE(m_hWideCharsBuff != NULL);
	}
	if (m_hMultiBytesBuff == NULL)
	{
		if ((m_hMultiBytesBuff = ::GlobalAlloc(MEMTYPE, dwMBBsize)) == NULL)
		{
			dwRetCode = ::GetLastError();
		}
		else
		{
			_ASSERTE(m_lpszMultiBytesBuff == NULL);
			if ((m_lpszMultiBytesBuff = (LPSTR)::GlobalLock(m_hMultiBytesBuff)) == NULL)
			{
				dwRetCode = ::GetLastError();
			}
			else
			{
				m_dwMultiBytesBuff = ::GlobalSize(m_hMultiBytesBuff);
				_ASSERTE(m_dwMultiBytesBuff != 0);
			}
			_ASSERTE(m_lpszMultiBytesBuff != NULL);

		}
		_ASSERTE(m_hMultiBytesBuff != NULL);
	}
	return dwRetCode;
}

DWORD CConvertCP::ReleaseBuffers()
{
	DWORD dwRetCode = 0;
	BOOL bRetVal = FALSE;
	if (m_hWideCharsBuff != NULL)
	{
		if (!(bRetVal = ::GlobalUnlock(m_hWideCharsBuff)))
		{
			if ((dwRetCode = ::GetLastError()) == NO_ERROR)
				bRetVal = TRUE;
		}
		if (bRetVal)
		{
			if (::GlobalFree(m_hWideCharsBuff) == NULL)
			{
				m_hWideCharsBuff = NULL;
				m_lpszWideCharsBuff = NULL;
				m_dwWideCharsBuff = 0;
			}
			else
				dwRetCode = ::GetLastError();
		}
		else
			dwRetCode = ::GetLastError();
		_ASSERTE(m_hWideCharsBuff == NULL);
	}
	if (dwRetCode == NO_ERROR && m_hMultiBytesBuff != NULL)
	{
		if (!(bRetVal = ::GlobalUnlock(m_hMultiBytesBuff)))
		{
			if ((dwRetCode = ::GetLastError()) == NO_ERROR)
				bRetVal = TRUE;
		}
		if (bRetVal)
		{
			if (::GlobalFree(m_hMultiBytesBuff) == NULL)
			{
				m_hMultiBytesBuff = NULL;
				m_lpszMultiBytesBuff = NULL;
				m_dwMultiBytesBuff = 0;
			}
			else
				dwRetCode = ::GetLastError();
		}
		else
			dwRetCode = ::GetLastError();
		_ASSERTE(m_hMultiBytesBuff == NULL);
	}
	if (dwRetCode != NO_ERROR)
		m_dwBufferLength = 0;
	return dwRetCode;
}

DWORD CConvertCP::ReAllocBuffer(DWORD dwNewSize, CConvertCP::BuffTypes BuffType, BOOL bApplyIncriment)
{
	DWORD dwRetCode = 0;
	BOOL bRetVal = FALSE;
	HGLOBAL hMem = NULL;
	_ASSERTE(dwNewSize > 0);
	switch (BuffType)
	{
	case WCBtype:
		if (bApplyIncriment)
			dwNewSize += m_dwBufferIncrement;
		_ASSERTE(m_hWideCharsBuff != NULL);
		if (!(bRetVal = ::GlobalUnlock(m_hWideCharsBuff)))
		{
			if ((dwRetCode = ::GetLastError()) == NO_ERROR)
				bRetVal = TRUE;
		}
		if (bRetVal)
		{
			if ((hMem = ::GlobalReAlloc(m_hWideCharsBuff, dwNewSize, GMEM_MOVEABLE|GMEM_ZEROINIT)) != NULL)
			{
				m_hWideCharsBuff = hMem;
				if ((m_lpszWideCharsBuff = (LPWSTR)::GlobalLock(m_hWideCharsBuff)) == NULL)
				{
					dwRetCode = ::GetLastError();
				}
				else
				{
					m_dwWideCharsBuff = ::GlobalSize(m_hWideCharsBuff);
					_ASSERTE(m_dwWideCharsBuff >= dwNewSize);
					if (m_dwWideCharsBuff > m_dwBufferLength)
						m_dwBufferLength = m_dwWideCharsBuff;
				}
			}
			else
				dwRetCode = ::GetLastError();
		}
		hMem = NULL;
		break;
	case MBBtype:
		if (bApplyIncriment)
			dwNewSize += m_dwBufferIncrement;
		_ASSERTE(m_hMultiBytesBuff != NULL);
		if (!(bRetVal = ::GlobalUnlock(m_hMultiBytesBuff)))
		{
			if ((dwRetCode = ::GetLastError()) == NO_ERROR)
				bRetVal = TRUE;
		}
		if (bRetVal)
		{
			if ((hMem = ::GlobalReAlloc(m_hMultiBytesBuff, dwNewSize, GMEM_MOVEABLE|GMEM_ZEROINIT)) != NULL)
			{
				m_hMultiBytesBuff = hMem;
				if ((m_lpszMultiBytesBuff = (LPSTR)::GlobalLock(m_hMultiBytesBuff)) == NULL)
				{
					dwRetCode = ::GetLastError();
				}
				else
				{
					m_dwMultiBytesBuff = ::GlobalSize(m_hMultiBytesBuff);
					_ASSERTE(m_dwMultiBytesBuff >= dwNewSize);
					if (m_dwMultiBytesBuff > m_dwBufferLength)
						m_dwBufferLength = m_dwMultiBytesBuff;
				}
			}
			else
				dwRetCode = ::GetLastError();
		}
		hMem = NULL;
		break;
	default :
		_ASSERTE(FALSE);
	}
	return dwRetCode;
}

STDMETHODIMP CConvertCP::get_BufferLength(long *pVal)
{
	if (pVal != NULL)
	{
		*pVal = (long)m_dwBufferLength;
		return S_OK;
	}
	else
		return E_INVALIDARG;
}

STDMETHODIMP CConvertCP::put_BufferLength(long newVal)
{
	if (newVal > 0)
	{
		DWORD dwErrCode = 0;
		if ((dwErrCode = ReAllocBuffer((DWORD)newVal, WCBtype, FALSE)) != NO_ERROR)
		{
			return Error(L"Can not ReAllocate WCB buffer", IID_IConvertCP, HRESULT_FROM_WIN32(dwErrCode));
		}
		if ((dwErrCode = ReAllocBuffer((DWORD)newVal, MBBtype, FALSE)) != NO_ERROR)
		{
			return Error(L"Can not ReAllocate MBB buffer", IID_IConvertCP, HRESULT_FROM_WIN32(dwErrCode));
		}
		m_dwBufferLength = (DWORD)newVal;
		return S_OK;
	}
	else
		return E_INVALIDARG;
}

STDMETHODIMP CConvertCP::get_BufferIncrement(long *pVal)
{
	if (pVal != NULL)
	{
		*pVal = (long)m_dwBufferIncrement;
		return S_OK;
	}
	else
		return E_INVALIDARG;
}

STDMETHODIMP CConvertCP::put_BufferIncrement(long newVal)
{
	if (newVal > 0)
	{
		m_dwBufferIncrement = (DWORD)newVal;
		return S_OK;
	}
	else
		return E_INVALIDARG;
}
