'///////////////////////////////////////////
' ConvertFile.vbs
' Usage:
'   ConvertFile.vbs [? | [<path>]EncodeFile [[<path>]DecodeFile [CpFrom [CpTo]]]]
'   Default Parameters:
'      EncodeFile - Encode.txt
'      DecodeFile - Decode.txt
'      CpFrom     - 866
'      CpTo       - 1251
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
' Author: Michael Drozdov
' Date:   07/19/2002
' Revision History:
'     Date         Comment
'    07/19/2002    Initial version started
'///////////////////////////////////////////

Option Explicit
On Error Resume Next

'
'-- Default Parameters
Const CPFORM 	= 866
Const CPTO 	= 1251
Const INPFILE = "Encode.txt"
Const OUTFILE = "Decode.txt"
Const SHOWRESULT = False
Const BUFFERLENGTH = 2048
Const BUFFERINCRIMANT = 1024

Dim oArgObj ' Object which contains the command line argument
Dim oCPconvert 'As CPCONVERTLib.ConvertCP
Dim oFso 'As Scripting.FileSystemObject
Dim oTsInp 'As TextStream
Dim oTsOut 'As TextStream
Dim strFileNameInp 'As String
Dim strFileNameOut 'As String
Dim nCpFrom 'As Integer
Dim nCpTo 'As Integer
Dim bShowResult 'As Boolean

Const ForReading = 1
Const ForWriting = 2
Const GENERAL_FAILURE = 2
Const GENERAL_WARNING = 1

'
'-- From/To Code Pages
nCpFrom = CPFORM
nCpTo = CPTO

'
'-- Input/Output files
strFileNameInp = INPFILE
strFileNameOut = OUTFILE

bShowResult = SHOWRESULT

Set oTsInp = Nothing
Set oTsOut = Nothing
Set oFso = Nothing
Set oCPconvert = Nothing
Set oArgObj = Nothing

'
'-- ConvertCP object
Set oCPconvert = Wscript.CreateObject("CPConvert.ConvertCP")
If oCPconvert Is Nothing Then
	WScript.Echo "Can not create 'CPConvert.ConvertCP' object"
	Call Cleanup(Err.Number)
End If

'
'-- Change BufferLength or/and BufferIncrement properties if need
oCPconvert.BufferLength = BUFFERLENGTH
If oCPconvert.BufferLength <> BUFFERLENGTH Then
	WScript.Echo "Can not change 'BufferLength' property"
	Call Cleanup(Err.Number)
End If
oCPconvert.BufferIncrement = BUFFERINCRIMANT
If oCPconvert.BufferIncrement <> BUFFERINCRIMANT Then
	WScript.Echo "Can not change 'BufferIncrement' property"
	Call Cleanup(Err.Number)
End If

'
'-- Get the Arguments object
Set oArgObj = WScript.Arguments
If oArgObj.Count > 0 Then
	Dim i 'As Integer
	For i = 0 To oArgObj.Count-1
		Select Case i
	    	Case 0
	    		strFileNameInp = oArgObj.Item(I)
	    		If InStr(strFileNameInp, "?") <> 0 Then
	    			Call DisplayHelpMessage()
	    			Call Cleanup(GENERAL_WARNING)
	    		End If
	    	Case 1
	    		strFileNameOut = oArgObj.Item(I)
	    	Case 2
	    		nCpFrom = CInt(oArgObj.Item(I))
	    	Case 3
	    		nCpTo = CInt(oArgObj.Item(I))
		End Select
	Next
End If
Set oArgObj = Nothing

'
'-- FileSystemObject
Set oFso = Wscript.CreateObject("Scripting.FileSystemObject")
If oFso Is Nothing Then
	WScript.Echo "Can not create 'Scripting.FileSystemObject' object"
	Call Cleanup(Err.Number)
End If

'
'-- Open input file
Set oTsInp = oFso.OpenTextFile(strFileNameInp, ForReading, False)
If oTsInp Is Nothing Then
	WScript.Echo "Can not open file: '" & strFileNameInp & "'"
	Call Cleanup(Err.Number)
End If

'
'-- Open output file
Set oTsOut = oFso.OpenTextFile(strFileNameOut, ForWriting, True)
If oTsOut Is Nothing Then
	WScript.Echo "Can not create/open file: '" & strFileNameOut & "'"
	Call Cleanup(Err.Number)
End If

'
'-- Convert oTsInp -> oTsOut
Dim strReadLineText 'As String
Dim nCountLine 'As Long
nCountLine = 0
Do While oTsInp.AtEndOfStream <> True
	strReadLineText = oTsInp.ReadLine
	Call oTsOut.WriteLine(oCPconvert.Convert(nCpFrom, nCpTo, strReadLineText))
	nCountLine = nCountLine + 1
Loop

'
'-- Show result
If bShowResult Then
	WScript.Echo nCountLine & " line(s) converted"
End If	

Call Cleanup(0)

Sub Cleanup(ErrNum)
	If Not oTsOut Is Nothing Then
		oTsOut.Close()
		Set oTsOut = Nothing
	End If	
	If Not oTsInp Is Nothing Then
		oTsInp.Close()
		Set oTsInp = Nothing
	End If	
	Set oCPconvert = Nothing
	Set oFso = Nothing
	Set oArgObj = Nothing
	WScript.Quit (ErrNum)
End Sub	

Sub DisplayHelpMessage()
	WScript.Echo "Usage:" & vbCrLf & _
		"ConvertFile.vbs [? | [<path>]EncodeFile [[<path>]DecodeFile" & _
		" [CpFrom [CpTo]]]]"
End Sub